#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Dynamically computed seasonality indicator for use on daily charts.
	/// Copyright www.Emini-Watch.com  All rights reserved. 
    /// </summary>
    [Description("Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.")]
    public class EminiWatch_DailySeasonalityIndicator : Indicator
    {
		
		private bool 		january01		= false;
		private	int			startYear;
		private	int 		yearNr  		= 0;
		private	int 		monthNr 		= 0;
		private	int 		dayNr   		= 0;
		private double[,,]	data			= new double[25,12,25];


        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "DailySeasonality"));
			Add(new Line(Color.White, 0, "Zero"));
            Overlay	= false;
			DrawOnPricePanel = false;
        }

		
        protected override void OnBarUpdate()
        {
			// Only use on day chart
			if (BarsPeriod.Id != PeriodType.Day)
				return;

			// We use Close[5], which means we need to skip first few bars
			// Chart should be set to start 1st January, or we just wait for (roughly) first January.
			if (CurrentBar < 5)
				return;
			if (!january01 && Time[0].Month == 1 && Time[1].Month != 1)
			{
				january01 = true;
				startYear = Time[0].Year;
			}
			if (!january01)
				return;		

			yearNr  = Time[0].Year - startYear;
			monthNr = Time[0].Month - 1;
			dayNr   = Time[0].Month != Time[1].Month ? 0 : dayNr + 1;

			double value;
			value = (((Close[0]/Close[5])-1)*100)-0.16;		// Factor of 0.16 used to adjust for long term uptrend;
			value = Math.Min(value,2);						// Austrlian market value is 0.14, equivalent to 7% pa;
			value = Math.Max(value,-2);						//  US market value is 0.18, equivalent to 10% pa

			data[yearNr, monthNr, dayNr] = value;

			double avg = 0.0;
			for (int year = 0; year <= yearNr; year++)
				avg += data[year,monthNr,dayNr];
			avg = avg/(yearNr+1);
			data[yearNr+1,monthNr,dayNr] = avg;

			Value.Set(avg);
			PlotColors[0][0] = (avg >= 0) ? Color.Green : Color.Red;

			if (CurrentBar >= Count - 2)
			{
				if (dayNr>=23) {monthNr++; dayNr = 0;}
				DrawDot("Proj1", true, -1, data[yearNr,monthNr,dayNr+1], Color.Blue);
				DrawDot("Proj2", true, -2, data[yearNr,monthNr,dayNr+2], Color.Blue);
				DrawDot("Proj3", true, -3, data[yearNr,monthNr,dayNr+3], Color.Blue);
			}
        }
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private EminiWatch_DailySeasonalityIndicator[] cacheEminiWatch_DailySeasonalityIndicator = null;

        private static EminiWatch_DailySeasonalityIndicator checkEminiWatch_DailySeasonalityIndicator = new EminiWatch_DailySeasonalityIndicator();

        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        public EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator()
        {
            return EminiWatch_DailySeasonalityIndicator(Input);
        }

        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        public EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator(Data.IDataSeries input)
        {
            if (cacheEminiWatch_DailySeasonalityIndicator != null)
                for (int idx = 0; idx < cacheEminiWatch_DailySeasonalityIndicator.Length; idx++)
                    if (cacheEminiWatch_DailySeasonalityIndicator[idx].EqualsInput(input))
                        return cacheEminiWatch_DailySeasonalityIndicator[idx];

            lock (checkEminiWatch_DailySeasonalityIndicator)
            {
                if (cacheEminiWatch_DailySeasonalityIndicator != null)
                    for (int idx = 0; idx < cacheEminiWatch_DailySeasonalityIndicator.Length; idx++)
                        if (cacheEminiWatch_DailySeasonalityIndicator[idx].EqualsInput(input))
                            return cacheEminiWatch_DailySeasonalityIndicator[idx];

                EminiWatch_DailySeasonalityIndicator indicator = new EminiWatch_DailySeasonalityIndicator();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                EminiWatch_DailySeasonalityIndicator[] tmp = new EminiWatch_DailySeasonalityIndicator[cacheEminiWatch_DailySeasonalityIndicator == null ? 1 : cacheEminiWatch_DailySeasonalityIndicator.Length + 1];
                if (cacheEminiWatch_DailySeasonalityIndicator != null)
                    cacheEminiWatch_DailySeasonalityIndicator.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheEminiWatch_DailySeasonalityIndicator = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator()
        {
            return _indicator.EminiWatch_DailySeasonalityIndicator(Input);
        }

        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        public Indicator.EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator(Data.IDataSeries input)
        {
            return _indicator.EminiWatch_DailySeasonalityIndicator(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator()
        {
            return _indicator.EminiWatch_DailySeasonalityIndicator(Input);
        }

        /// <summary>
        /// Dynamically computed seasonality indicator for use on daily charts. Needed is at least 1+ years of data. Copyright www.Emini-Watch.com  All rights reserved.
        /// </summary>
        /// <returns></returns>
        public Indicator.EminiWatch_DailySeasonalityIndicator EminiWatch_DailySeasonalityIndicator(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.EminiWatch_DailySeasonalityIndicator(input);
        }
    }
}
#endregion
